#include <iostream>
#include <iomanip>
#include <vector>
#include <cmath>
#include <sstream>
#include <cstring>
#include <concepts>

// Концепт для проверки, что тип имеет метод module(), возвращающий значение, преобразуемое в double
template<typename T>
concept HasModule = requires(T a) {
    { a.module() } -> std::convertible_to<double>;
};

namespace linalg {

    // Шаблонный класс Matrix для работы с матрицами произвольного типа данных T
    template<typename T>
    class Matrix {
    public:

        // Объявление дружественного класса Matrix для другого типа данных
        template<typename U>
        friend
        class Matrix;

        // Дружественная функция для сравнения двух матриц разных типов
        template<typename U, typename Y>
        friend bool operator==(const Matrix<U> &first, const Matrix<Y> &second) noexcept;

        // Оператор присваивания для матриц одного типа
        Matrix &operator=(const Matrix &other) { return operator=<T>(other); }

        // Конструктор по умолчанию
        Matrix() = default;

        // Конструктор для создания матрицы с заданным количеством строк
        Matrix(size_t rows);

        // Конструктор для создания матрицы с заданным количеством строк и столбцов
        Matrix(size_t rows, size_t columns);

        // Запрещаем копирование матрицы с модификатором volatile
        Matrix(const volatile Matrix &) = delete;

        // Шаблонный конструктор копирования для матриц разных типов
        template<typename U>
        Matrix(const Matrix<U> &other);

        // Шаблонный перемещающий конструктор для матриц разных типов
        template<typename U>
        Matrix(Matrix<U> &&other) noexcept;

        // Конструктор инициализации из одномерного списка значений
        template<typename U>
        Matrix(std::initializer_list<U> lst);

        // Конструктор инициализации из двумерного списка значений
        template<typename U>
        Matrix(std::initializer_list<std::initializer_list<U>> lst);

        // Деструктор
        ~Matrix() noexcept;


        // Шаблонный оператор присваивания для матриц разных типов
        template<typename U>
        Matrix &operator=(const Matrix<U> &other);

        // Шаблонный оператор перемещающего присваивания для матриц разных типов
        template<typename U>
        Matrix &operator=(Matrix<U> &&other) noexcept;

        size_t size() const noexcept { return m_rows * m_columns; }

        void swap(Matrix& other) noexcept;

        T& operator()(size_t i, size_t j) noexcept { return m_ptr[i * m_columns + j]; }

        const T& operator()(size_t i, size_t j) const noexcept { return m_ptr[i * m_columns + j]; }

        template<typename U>
        Matrix& operator+=(const Matrix<U>& other);

        template<typename U>
        Matrix& operator-=(const Matrix<U>& other);

        template<typename U>
        Matrix& operator*=(const Matrix<U>& other);

        template<typename U>
        Matrix& operator*=(U num) noexcept;

        size_t rows() const noexcept { return m_rows; }

        size_t columns() const noexcept { return m_columns; }

        void reserve(size_t new_cap) noexcept;

        size_t capacity() const noexcept { return m_capacity; }

        bool empty() const noexcept { return m_ptr == nullptr; }

        void reshape(size_t rows, size_t columns);

        void shrink_to_fit() noexcept;

        void clear() { m_columns = m_rows = 0; }

        T* begin() { return m_ptr; };

        T* end() { return m_ptr + (size()); }

        long double norm() const noexcept;

        T trace() const;

        T det() const;


        Matrix inversed() const;

        Matrix transposed() const;

    private:

        // Указатель на данные матрицы
        T *m_ptr = nullptr;
        // Количество строк в матрице
        size_t m_rows = 0;
        // Количество столбцов в матрице
        size_t m_columns = 0;
        // Емкость выделенной памяти для элементов матрицы
        size_t m_capacity = 0;

        // Вспомогательный метод для освобождения памяти
        template<typename U>
        void free(U *&ptr, size_t n) noexcept;

        // Метод для вычисления определителя, объявление без реализации (реализация будет ниже).
        T recursive_determinant(const T* ptr, size_t n) const;

//  Шаблонная функция для сравнения двух значений произвольных типов (U и Y).

        template<typename U, typename Y>
        static bool are_equal(const U& a, const Y& b) {
            return a == b; // Возвращает true, если значения равны.
        }

//  Шаблонная перегрузка для сравнения числа с плавающей запятой (double) и значения произвольного типа (U).
// Сравнение выполняется с учетом допустимой погрешности epsilon.
        template<typename U>
        static bool are_equal(const double& a, const U& b, double epsilon = 1e-9) {
            // Сравнивает абсолютную разницу значений с epsilon.
            return std::fabs(a - b) < epsilon;
        }

//  Шаблонная перегрузка для сравнения значения произвольного типа (U) и числа с плавающей запятой (double).

        template<typename U>
        static bool are_equal(const U& a, const double& b, double epsilon = 1e-9) {
            // Аналогично предыдущей функции, сравнивает разницу.
            return std::fabs(a - b) < epsilon;
        }

//  Статическая функция для вычисления модуля (abs) значения с пользовательской реализацией метода `module()`.

        static double abs(const HasModule auto& value) {
            return value.module(); // Возвращает результат пользовательского метода module().
        }

//  Перегрузка для вычисления модуля стандартных типов.

        static double abs(const auto& value) {
            return std::abs(value); // Возвращает модуль числа.
        }

    };

    template<typename T>
// Функция swap для обмена двух матриц местами (реализует семантику перемещения)
    void swap(Matrix<T> &first, Matrix<T> &second) noexcept { first.swap(second); }

// Конструктор для создания матрицы с заданным количеством строк (по умолчанию 1 столбец)
    template<typename T>
    Matrix<T>::Matrix(size_t rows) : m_rows(rows), m_columns(1), m_capacity(rows) {
        // Выделяем память для элементов матрицы
        m_ptr = reinterpret_cast<T *>(new std::byte[m_capacity * sizeof(T)]);

        size_t i = 0;

        try {
            // Инициализируем элементы матрицы значениями по умолчанию
            for (; i < rows; ++i) {
                new(m_ptr + i) T();
            }
        } catch (...) {
            // Освобождаем память в случае исключения
            free<T>(m_ptr, i + 1);
            this->~Matrix();
            throw;
        }
    }

// Конструктор для создания матрицы с заданным количеством строк и столбцов
    template<typename T>
    Matrix<T>::Matrix(size_t rows, size_t columns): m_rows(rows), m_columns(columns), m_capacity(rows * columns) {
        // Выделяем память для элементов матрицы
        m_ptr = reinterpret_cast<T *>(new std::byte[m_capacity * sizeof(T)]);

        size_t i = 0;

        try {
            // Инициализируем элементы матрицы значениями по умолчанию
            for (; i < rows * columns; ++i) {
                new(m_ptr + i) T();
            }
        } catch (...) {
            // Освобождаем память в случае исключения
            free<T>(m_ptr, i + 1);
            this->~Matrix();
            throw;
        }
    }

// Конструктор копирования для создания матрицы из другой матрицы другого типа
    template<typename T>
    template<typename U>
    Matrix<T>::Matrix(const Matrix<U> &other) : m_rows(other.m_rows), m_columns(other.m_columns),
                                                m_capacity(m_rows * m_columns) {
        // Выделяем память для новой матрицы
        m_ptr = reinterpret_cast<T *>(new std::byte[m_capacity * sizeof(T)]);

        size_t i = 0;

        try {
            // Копируем элементы из другой матрицы
            for (; i < m_rows * m_columns; ++i) {
                new(m_ptr + i) T((other.m_ptr[i]));
            }
        } catch (...) {
            // Освобождаем память в случае исключения
            free<T>(m_ptr, i + 1);
            this->~Matrix();
            throw;
        }
    }

// Перемещающий конструктор для создания матрицы из другой матрицы, переданной по rvalue-ссылке
    template<typename T>
    template<typename U>
    Matrix<T>::Matrix(Matrix<U> &&other) noexcept : m_ptr(reinterpret_cast<T *>(other.m_ptr)), m_rows(other.m_rows),
                                                    m_columns(other.m_columns), m_capacity(other.m_capacity) {
        // Обнуляем исходную матрицу, чтобы она не владела ресурсами
        other.m_ptr = nullptr;
        other.m_columns = other.m_rows = other.m_capacity = 0;
    }

// Конструктор инициализации из одномерного списка значений
    template<typename T>
    template<typename U>
    Matrix<T>::Matrix(std::initializer_list<U> lst) : m_rows(lst.size()), m_columns(1), m_capacity(lst.size()) {
        // Выделяем память для элементов матрицы
        m_ptr = reinterpret_cast<T *>(new std::byte[m_capacity * sizeof(T)]);

        size_t i = 0;

        try {
            auto iter = lst.begin();
            // Копируем значения из списка
            for (; i < lst.size(); ++i) {
                new(m_ptr + i) T(*iter);
                ++iter;
            }
        } catch (...) {
            // Освобождаем память в случае исключения
            free<T>(m_ptr, i + 1);
            this->~Matrix();
            throw;
        }
    }

// Конструктор инициализации из двумерного списка значений
    template<typename T>
    template<typename U>
    Matrix<T>::Matrix(std::initializer_list<std::initializer_list<U>> lst) : m_rows(lst.size()),
                                                                             m_columns(lst.begin()->size()),
                                                                             m_capacity(m_rows * m_columns) {
        // Выделяем память для элементов матрицы
        m_ptr = reinterpret_cast<T *>(new std::byte[m_capacity * sizeof(T)]);

        size_t i = 0;

        try {
            // Копируем значения из двумерного списка
            for (const auto &row : lst) {
                for (auto &elem : row) {
                    new(m_ptr + i) T(elem);
                    ++i;
                }
            }
        } catch (...) {
            // Освобождаем память в случае исключения
            free<T>(m_ptr, i + 1);
            this->~Matrix();
            throw;
        }
    }

// Деструктор матрицы для освобождения ресурсов
    template<typename T>
    Matrix<T>::~Matrix() noexcept {
        if (m_ptr != nullptr) {
            // Освобождаем память для всех элементов матрицы
            free<T>(m_ptr, size());
        }
    }

// Оператор копирующего присваивания для матриц разных типов
    template<typename T>
    template<typename U>
    Matrix<T> &Matrix<T>::operator=(const Matrix<U> &other) {
        if (&other == this) {
            return *this;
        }

        // Если емкость текущей матрицы позволяет, обновляем существующие элементы
        if (m_capacity >= other.size()) {
            try {
                for (size_t i = 0; i < size(); ++i) {
                    m_ptr[i] = other.m_ptr[i];
                }
                for (size_t i = size(); i < other.size(); ++i) {
                    new(m_ptr + i) T(other.m_ptr[i]);
                }
            } catch (...) {
                throw;
            }
            for (size_t i = other.size(); i < size(); ++i) {
                (m_ptr + i)->~T();
            }
            m_columns = other.m_columns;
            m_rows = other.m_rows;
            m_capacity = size();
            return *this;
        }

        // Если емкость недостаточна, создаем копию матрицы и обмениваемся ресурсами
        try {
            Matrix<T> copy(other);
            ::linalg::swap(copy, *this);
        } catch (...) {
            throw;
        }

        return *this;
    }

// Оператор перемещающего присваивания для матриц разных типов
    template<typename T>
    template<typename U>
    Matrix<T> &Matrix<T>::operator=(Matrix<U> &&other) noexcept {
        if (&other == this) {
            return *this;
        }

        // Уничтожаем текущие элементы матрицы
        for (size_t i = 0; i < size(); ++i) {
            (m_ptr + i)->~T();
        }

        // Освобождаем выделенную память
        delete[] reinterpret_cast<std::byte *>(m_ptr);
        m_rows = other.m_rows;
        m_columns = other.m_columns;
        m_ptr = reinterpret_cast<T *>(other.m_ptr);

        // Обнуляем другую матрицу
        other.m_ptr = nullptr;
        other.m_columns = other.m_rows = 0;

        return *this;
    }
    template<typename T>
// Оператор += для поэлементного сложения матриц
    template<typename U>
    Matrix<T>& Matrix<T>::operator+=(const Matrix<U>& other) {
        // Проверяем совместимость размеров матриц
        if (m_columns != other.m_columns || m_rows != other.m_rows) {
            throw std::runtime_error("Incompatible sizes");
        }

        // Поэлементно складываем значения
        for (size_t i = 0; i < size(); ++i) {
            m_ptr[i] += other.m_ptr[i];
        }

        return *this;
    }
    template<typename T>
    void Matrix<T>::swap(Matrix<T>& other) noexcept {
        std::swap(m_rows, other.m_rows);
        std::swap(m_columns, other.m_columns);
        std::swap(m_ptr, other.m_ptr);
        std::swap(m_capacity, other.m_capacity);
    }

    template<typename T>
// Оператор -= для поэлементного вычитания матриц
    template<typename U>
    Matrix<T>& Matrix<T>::operator-=(const Matrix<U>& other) {
        // Проверяем совместимость размеров матриц
        if (m_columns != other.m_columns || m_rows != other.m_rows) {
            throw std::runtime_error("Incompatible sizes");
        }

        // Поэлементно вычитаем значения
        for (size_t i = 0; i < size(); ++i) {
            m_ptr[i] -= other.m_ptr[i];
        }

        return *this;
    }

    template<typename T>
// Оператор *= для умножения матриц
    template<typename U>
    Matrix<T>& Matrix<T>::operator*=(const Matrix<U>& other) {
        // Проверяем, что число столбцов текущей матрицы совпадает с числом строк другой матрицы
        if (m_columns != other.m_rows) {
            throw std::runtime_error("Incompatible sizes");
        }

        // Создаем временную матрицу для хранения результата
        Matrix<T> res;
        try {
            res = Matrix<T>(m_rows, other.m_columns);
        } catch(...) {
            throw;
        }

        // Выполняем умножение матриц
        for (size_t i = 0; i < m_rows; ++i) {
            for (size_t j = 0; j < other.m_columns; ++j) {
                for (size_t k = 0; k < m_columns; ++k) {
                    res(i,j) += (*this)(i, k) * other(k,j);
                }
            }
        }

        // Присваиваем результат текущей матрице
        *this = std::move(res);
        return *this;
    }

    template<typename T>
// Оператор *= для умножения матрицы на скаляр
    template<typename U>
    Matrix<T>& Matrix<T>::operator*=(U num) noexcept {
        // Умножаем каждый элемент матрицы на скаляр
        for (size_t i = 0; i < size(); ++i) {
            m_ptr[i] *= num;
        }

        return *this;
    }

    template<typename T>
    void Matrix<T>::reserve(size_t new_cap) noexcept {
        // Если новый объем (new_cap) меньше или равен текущей емкости (m_capacity),
        // то нет необходимости в перераспределении памяти.
        if (new_cap <= m_capacity) {
            return;
        }

        // Обновляем значение емкости.
        m_capacity = new_cap;

        // Выделяем новую память для хранения объектов типа T.
        // Используется reinterpret_cast, так как выделяется массив сырых байтов.
        T* new_ptr = reinterpret_cast<T*>(new std::byte[new_cap * sizeof(T)]);

        // Копируем данные из старого массива в новый массив.
        std::memcpy(new_ptr, m_ptr, size() * sizeof(T));

        // Освобождаем старую память, предварительно преобразовав указатель обратно к byte*.
        delete[] reinterpret_cast<std::byte*>(m_ptr);

        // Обновляем указатель на новый массив.
        m_ptr = new_ptr;
    }


    template<typename T>
    void Matrix<T>::reshape(size_t rows, size_t columns) {
        // Если новый размер матрицы меньше текущего количества элементов,
        // уничтожаем лишние объекты вручную.
        if (rows * columns < size()) {
            for (size_t i = rows * columns; i < size(); ++i) {
                (m_ptr + i)->~T(); // Вызываем деструктор для лишних объектов.
            }
        }
            // Если новый размер больше текущего, то:
        else if (rows * columns > size()) {
            // Увеличиваем емкость, если требуется.
            reserve(rows * columns);

            // Инициализируем новые элементы, используя конструктор по умолчанию.
            try {
                for (size_t i = size(); i < rows * columns; ++i) {
                    new (m_ptr + i) T(); // Размещение нового объекта.
                }
            } catch (...) {
                // Если возникает исключение при создании объектов,
                // уничтожаем текущий объект Matrix и пробрасываем исключение дальше.
                this->~Matrix();
                throw;
            }
        }

        // Обновляем размеры матрицы.
        m_rows = rows;
        m_columns = columns;
    }

    template<typename T>
    T Matrix<T>::det() const {
        if (m_columns != m_rows || m_columns * m_rows == 0) {
            throw std::runtime_error("Not a square matrix");
        }
        return Matrix::recursive_determinant(m_ptr, m_rows);
    }

    template<typename T>
    T Matrix<T>::recursive_determinant(const T* ptr, size_t n) const {
        // Если размер матрицы 1x1, возвращаем единственный элемент.
        if (n == 1) {
            return ptr[0];
        }

        T det = 0; // Переменная для накопления значения определителя.

        try {
            // Создаем вектор для хранения подматрицы размером (n-1)x(n-1).
            std::vector<T> submatrix((n - 1) * (n - 1));

            // Перебираем элементы первой строки матрицы.
            for (int k = 0; k < n; ++k) {
                // Заполняем подматрицу, исключая текущую строку (i = 0) и столбец (j = k).
                for (int i = 1; i < n; ++i) {
                    int sub_i = i - 1; // Смещение индекса строки подматрицы.
                    for (int j = 0, sub_j = 0; j < n; ++j) {
                        if (j == k) continue; // Пропускаем столбец k.
                        submatrix[sub_i * (n - 1) + sub_j] = ptr[i * n + j];
                        ++sub_j; // Смещение индекса столбца подматрицы.
                    }
                }

                // Определяем знак разложения (чередуется: +, -, +, ...).
                T sign = (k % 2 == 0) ? 1 : -1;

                // Рекурсивно вычисляем определитель подматрицы и добавляем вклад в общий определитель.
                det += sign * ptr[k] * recursive_determinant(submatrix.data(), n - 1);
            }
        } catch(...) {
            // Перехват исключений на случай ошибок, например, недостатка памяти.
            throw;
        }

        return det; // Возвращаем вычисленный определитель.
    }
    // Метод для вычисления нормы матрицы.
    template<typename T>
    long double Matrix<T>::norm() const noexcept {
        double res = 0;

        // Вычисляем сумму квадратов всех элементов матрицы.
        for (size_t i = 0; i < size(); ++i) {
            res += m_ptr[i] * m_ptr[i];
        }

        // Возвращаем квадратный корень из суммы (евклидова норма).
        return pow(res, 0.5);
    }

// Метод для вычисления следа матрицы (суммы элементов на главной диагонали).
    template<typename T>
    T Matrix<T>::trace() const {
        // Проверяем, что матрица является квадратной и не пустой.
        if (m_columns != m_rows || m_columns * m_rows == 0) {
            throw std::runtime_error("Not a square matrix");
        }

        T res = 0;

        // Суммируем элементы на главной диагонали.
        for (size_t i = 0; i < m_rows; ++i) {
            res += m_ptr[i * m_columns + i]; // Элемент диагонали находится по индексу [i, i].
        }

        return res;
    }

// Шаблонный метод для освобождения памяти, занятой массивом объектов.
    template<typename T>
    template<typename U>
    void Matrix<T>::free(U*& ptr, size_t n) noexcept {
        // Уничтожаем все объекты в массиве, вызывая их деструкторы.
        for (size_t i = 0; i < n; ++i) {
            (ptr + i)->~U();
        }

        // Освобождаем память, преобразуя указатель в байты.
        delete[] reinterpret_cast<std::byte*>(ptr);

        // Устанавливаем указатель в nullptr для безопасности.
        ptr = nullptr;
    }



// Метод для уменьшения емкости матрицы до текущего размера.
    template<typename T>
    void Matrix<T>::shrink_to_fit() noexcept {
        // Если емкость уже совпадает с размером, ничего не делаем.
        if (m_capacity == size()) {
            return;
        }

        // Обновляем значение емкости.
        m_capacity = size();

        // Выделяем новую память, достаточную для текущего размера матрицы.
        T* new_ptr = reinterpret_cast<T*>(new std::byte[size() * sizeof(T)]);

        // Копируем текущие элементы в новую память.
        std::memcpy(new_ptr, m_ptr, size() * sizeof(T));

        // Освобождаем старую память.
        delete[] reinterpret_cast<std::byte*>(m_ptr);

        // Обновляем указатель на новый массив.
        m_ptr = new_ptr;
    }
    // Метод для получения транспонированной матрицы.
    template <typename T>
    Matrix<T> Matrix<T>::transposed() const {
        Matrix<T> transposed; // Создаем объект для транспонированной матрицы.

        try {
            // Инициализируем транспонированную матрицу размером (m_columns x m_rows).
            transposed = Matrix<T>(m_columns, m_rows);

            // Проходим по всем элементам исходной матрицы.
            for (size_t i = 0; i < m_rows; ++i) {
                for (size_t j = 0; j < m_columns; ++j) {
                    // Записываем элемент (i, j) в (j, i) в транспонированной матрице.
                    transposed(j, i) = operator()(i, j);
                }
            }
        } catch(...) {
            // Если возникает исключение, пробрасываем его дальше.
            throw;
        }

        return transposed; // Возвращаем транспонированную матрицу.
    }

// Метод для получения обратной матрицы (если она существует).
    template <typename T>
    Matrix<T> Matrix<T>::inversed() const {
        size_t n = m_rows;

        // Проверяем, что матрица квадратная.
        if (n != m_columns) {
            throw std::runtime_error("Matrix is not square, cannot compute inverse.");
        }

        // Вычисляем определитель матрицы.
        T det = this->det();

        // Если определитель равен нулю, матрица вырожденная, и обратной не существует.
        if (det == T(0)) {
            throw std::runtime_error("Matrix is singular, cannot compute inverse.");
        }

        Matrix<T> augmented; // Создаем расширенную матрицу.
        try {
            // Инициализируем расширенную матрицу размером (n x 2n).
            augmented = Matrix<T>(n, 2 * n);

            // Формируем расширенную матрицу: копируем исходную матрицу и добавляем единичную справа.
            for (int i = 0; i < n; ++i) {
                for (int j = 0; j < n; ++j) {
                    augmented(i, j) = operator()(i, j);         // Исходная матрица.
                    augmented(i, j + n) = (i == j) ? T(1) : T(0); // Единичная матрица.
                }
            }
        } catch(...) {
            throw; // Пробрасываем исключение при ошибке.
        }

        // Прямой ход метода Гаусса для приведения к ступенчатому виду.
        for (int i = 0; i < n; ++i) {
            // Находим строку с максимальным элементом в текущем столбце (для численной устойчивости).
            int max_row = i;
            for (int j = i + 1; j < n; ++j) {
                if (abs(augmented(j, i)) > abs(augmented(max_row, i))) {
                    max_row = j;
                }
            }

            // Меняем текущую строку и строку с максимальным элементом местами.
            for (int j = 0; j < 2 * n; ++j) {
                std::swap(augmented(i, j), augmented(max_row, j));
            }

            // Проверяем, что главный элемент (пивот) не равен нулю.
            T pivot = augmented(i, i);
            if (pivot == T(0)) {
                throw std::invalid_argument("Matrix is singular, cannot compute inverse.");
            }

            // Нормируем текущую строку, деля все элементы на главный элемент.
            for (int j = 0; j < 2 * n; ++j) {
                augmented(i, j) /= pivot;
            }

            // Зануляем элементы в текущем столбце во всех других строках.
            for (int k = 0; k < n; ++k) {
                if (k != i) {
                    T factor = augmented(k, i);
                    for (int j = 0; j < 2 * n; ++j) {
                        augmented(k, j) -= factor * augmented(i, j);
                    }
                }
            }
        }

        Matrix<T> inv; // Создаем матрицу для хранения обратной матрицы.
        try {
            // Инициализируем обратную матрицу размером (n x n).
            inv = Matrix<T>(n, n);

            // Из расширенной матрицы копируем правую часть (n столбцов), которая стала обратной матрицей.
            for (int i = 0; i < n; ++i) {
                for (int j = 0; j < n; ++j) {
                    inv(i, j) = augmented(i, j + n);
                }
            }
        } catch(...) {
            throw; // Пробрасываем исключение при ошибке.
        }

        return inv; // Возвращаем обратную матрицу.
    }

// Оператор сложения двух матриц с потенциально разными типами элементов
template<typename T, typename U>
Matrix<std::common_type_t<T, U>> operator+(const Matrix<T>& first, const Matrix<U>& second) {
    // Создаём новую матрицу с общим типом элементов между T и U
    Matrix<std::common_type_t<T, U>> new_matr;
    try {
        // Пытаемся скопировать первую матрицу в новую матрицу
        new_matr = Matrix<std::common_type_t<T, U>>(first);
    } catch(...) {
        // Если при копировании произошла ошибка, пробрасываем исключение дальше
        throw;
    }
    // Прибавляем ко вновь созданной матрице вторую матрицу
    new_matr += second;
    // Возвращаем результирующую матрицу
    return new_matr;
}

// Оператор вычитания двух матриц с потенциально разными типами элементов
template<typename T, typename U>
Matrix<std::common_type_t<T, U>> operator-(const Matrix<T>& first, const Matrix<U>& second) {
    // Создаём новую матрицу с общим типом элементов между T и U
    Matrix<std::common_type_t<T, U>> new_matr;
    try {
        // Пытаемся скопировать первую матрицу в новую матрицу
        new_matr = Matrix<std::common_type_t<T, U>>(first);
    } catch(...) {
        
        throw;
    }
   
    new_matr -= second;
 
    return new_matr;
}
// Оператор умножения двух матриц с потенциально разными типами элементов
template<typename T, typename U>
Matrix<std::common_type_t<T, U>> operator*(const Matrix<T>& first, const Matrix<U>& second) {
    // Создаём новую матрицу с общим типом элементов между T и U
    Matrix<std::common_type_t<T, U>> new_matr;
    try {
        // Пытаемся скопировать первую матрицу в новую матрицу
        new_matr = Matrix<std::common_type_t<T, U>>(first);
    } catch(...) {
       
        throw;
    }
    
    new_matr *= second;
    
    return new_matr;
}

// Оператор умножения числа на матрицу (число слево, матрица справа)
template<typename T, typename U>
Matrix<std::common_type_t<T, U>> operator*(T num, const Matrix<U>& first) {
    // Создаём новую матрицу с общим типом элементов между T и U
    Matrix<std::common_type_t<T, U>> new_matr;
    try {
        // Пытаемся скопировать первую матрицу в новую матрицу
        new_matr = Matrix<std::common_type_t<T, U>>(first);
    } catch(...) {
       
        throw;
    }
    
    new_matr *= num;
   
    return new_matr;
}

// Оператор умножения матрицы на число (матрица слево, число справа)
template<typename T, typename U>
Matrix<std::common_type_t<T, U>> operator*(const Matrix<T>& first, U num) {
    // Создаём новую матрицу с общим типом элементов между T и U
    Matrix<std::common_type_t<T, U>> new_matr;
    try {
        // Пытаемся скопировать первую матрицу в новую матрицу
        new_matr = Matrix<std::common_type_t<T, U>>(first);
    } catch(...) {
        
        throw;
    }
  
    new_matr *= num;
 
    return new_matr;
}
template<typename T, typename U>
bool operator==(const Matrix<T>& first, const Matrix<U>& second) noexcept {
    // Проверяем, совпадают ли размеры матриц
    if (first.columns() != second.columns() || first.rows() != second.rows()) {
        return false;
    }

    // Проходим по всем элементам матрицы и сравниваем их
    for (size_t i = 0; i < first.rows(); ++i) {
        for(size_t j = 0; j < first.columns(); ++j) {
            // Используем статический метод are_equal для сравнения элементов
            if (!Matrix<T>::template are_equal<T, U>(first(i, j), second(i, j))) {
                return false;
            }
        }
    }

    return true; // Если все элементы равны, возвращаем true
}

template<typename T, typename U>
bool operator!=(const Matrix<T>& first, const Matrix<U>& second) noexcept {
    // Оператор "не равно" реализуется через отрицание оператора "равно"
    return !(first == second);
}

template<typename T>
std::ostream& operator<<(std::ostream& stream, const Matrix<T>& matr) noexcept {
    size_t rows = matr.rows();
    if (rows == 0) {
        return stream; // Если строк нет, просто возвращаем поток
    }

    size_t cols = matr.columns();
    std::vector<int> col_widths(cols, 0); // Вектор для хранения максимальной ширины каждого столбца
    std::vector<std::vector<int>> all_widths(rows, std::vector<int>(cols)); // Матрица для хранения ширины каждого элемента

    std::stringstream buf;
    std::string f;

    // Определяем необходимую ширину для каждого столбца
    for (size_t j = 0; j < cols; ++j) {
        for (size_t i = 0; i < rows; ++i) {
            buf << matr(i, j); // Записываем элемент в буфер
            buf >> f; // Считываем строковое представление элемента
            buf.clear(); // Очищаем буфер
            int width = f.size(); // Определяем ширину элемента
            all_widths[i][j] = width;
            if (width > col_widths[j]) {
                col_widths[j] = width; // Обновляем максимальную ширину столбца
            }
        }
    }

    // Выводим матрицу с учетом выравнивания
    for (size_t i = 0; i < rows; ++i) {
        stream << '|';
        for (size_t j = 0; j < cols; ++j) {
            // Добавляем отступы слева, чтобы выровнять столбцы
            stream << std::string(col_widths[j] - all_widths[i][j], ' ') << matr(i, j);
            if (j < cols - 1) {
                stream << ' '; // Добавляем пробел между элементами
            }
        }
        stream << '|' << '\n'; // Закрываем строку матрицы
    }

    return stream;
}

}
